mod cam_comm;
use std::{ fs, path::Path};
use chrono::{DateTime, Local};
use communication_layer::rest_api;
use log::LevelFilter;
use fern::{log_file, Dispatch};
use sudo::escalate_if_needed;


#[rocket::main]
async fn main() {
    setup_logs(&true);
    _ = escalate_if_needed();
    rest_api::start_rest_endpoint().await;
}


///Set up logging macros to be used throughout the program
fn setup_logs(debug:&bool){
    //Get the current time
    let chrono_now: DateTime<Local> = Local::now();
    //Create a log directory, if it does not exist
    if ! Path::new("logs").is_dir(){
        _ = fs::create_dir("logs");
    };
    //Create a log macro listener
    _ = fern::Dispatch::new()
        //Format the output messages
        .format(|out,message,record|{
            out.finish(format_args!(
                "{} - [{}, {}] - {}",
                Local::now().to_rfc3339(),
                record.level(),
                record.target(),
                message
            ))
        })
        //Output to the file
        .chain({
            let mut file_logger = Dispatch::new();
            let date_format = chrono_now.format("%Y-%m-%d_%H.%M").to_string();
            let local_log_file = log_file(format!("logs/{}.log",date_format)).unwrap();
            //Set filter based on whether the command is being run in "debug" mode
            if *debug{
                file_logger = file_logger.level(LevelFilter::Trace);
            }
            else {
                file_logger = file_logger.level(LevelFilter::Debug);
            }
            //Apply the filter and file formatting rules described above
            file_logger.chain(local_log_file)
        })
        .chain({
            //Also log to stdout
            let mut stdout_logger = fern::Dispatch::new();
            if *debug {
                stdout_logger = stdout_logger.level(LevelFilter::Debug);
            }
            else {
                stdout_logger = stdout_logger.level(LevelFilter::Info);
            }
                stdout_logger.chain(std::io::stdout())
        })
        //Start listening to macros
        .apply();
}
