use std::{path::{Path, PathBuf}, sync::Mutex};
use crate::cam_comm::WorldServerConnection;
use local_ip_address::local_ip;
use rocket::{figment::Figment, fs::NamedFile, http::Method, response::status, Config, State};
use rocket_cors::{AllowedOrigins, CorsOptions};

#[post("/serve", data = "<raw_index>")]
fn start_serve(raw_index:String,/* cams:&State<Mutex<WorldServerConnection>>*/) -> status::Accepted<()> {
    println!("{:?}",raw_index);
    //let mut lock = cams.lock().unwrap();
    //lock.get_next_images(index);
    status::Accepted(())
}

//Get most recently updated file
#[get("/serve")]
async fn get_serve_data() -> Option<NamedFile>{
    let last_modified_file = std::fs::read_dir(".")         //Read in the current directory
        .expect("couldn't access local dir")                //assume the directory is real
        .flatten()                                          //Ignore all the files that have permissions issues
        .filter(|file| file.metadata().unwrap().is_file())  //Ignore files that aren't actually files 
        .max_by_key(|x| x.metadata().unwrap().modified().unwrap()) //Find newest file
        .unwrap();  //Assume the newest file actually exists
    NamedFile::open(last_modified_file.path()).await.ok()
}

///Get specific file, as defined by request
#[deprecated(since = "TBD")]
#[get("/serve/<file..>")]
async fn get_data(file:PathBuf) -> Option<NamedFile>{
    NamedFile::open(file).await.ok()
}

///Initialises the world server connection, spins Rocket into a unique process
pub async fn start_rest_endpoint(){
    //Rocket defaults to using the localhost loopback address. This isn't ideal for prod usage, so
    //instead we bind to the host IP address.
    let host_ip = local_ip().expect("No real IP address!");
    let figment = Figment::from(Config::default())
        .merge(("address",host_ip));

    //Allow cross-origin resource sharing 
    let cors = CorsOptions::default()
        .allowed_origins(AllowedOrigins::all())
        .allowed_methods(
            vec![Method::Get, Method::Post]
            .into_iter()
            .map(From::from)
            .collect(),
        ).allow_credentials(true);

    let _rocket = rocket::custom(figment)
        .attach(cors.to_cors().expect("CORS init failure!"))
        //.manage(Mutex::new(WorldServerConnection::new().unwrap()))
        .mount("/", routes![get_data,start_serve,get_serve_data])
        .launch().await;
}
